from .base import Base


class Categories(Base):
    """Class for Categories APIs."""

    def __init__(self):
        Base.__init__(self, attribute_type='CATEGORY')

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        generator = self.get_all_generator()
        for items in generator:
            category_attributes = []
            for category in items['data']:
                if category['sub_category'] and category['name'] != category['sub_category']:
                    category['name'] = '{0} / {1}'.format(category['name'], category['sub_category'])

                category_attributes.append({
                    'attribute_type': self.attribute_type,
                    'display_name': self.attribute_type.replace('_', ' ').title(),
                    'value': category['name'],
                    'source_id': category['id'],
                    'active': category['is_enabled'],
                    'detail': None
                })

            self.bulk_create_or_update_expense_attributes(category_attributes, True)



from .base import Base


class Employees(Base):
    """Class for Employees APIs."""

    def __init__(self):
        Base.__init__(self, attribute_type='EMPLOYEE', query_params={'is_enabled': 'eq.true', 'has_accepted_invite': 'eq.true'})

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        generator = self.get_all_generator()
        for items in generator:
            employee_attributes = []
            for employee in items['data']:
                employee_attributes.append({
                    'attribute_type': self.attribute_type,
                    'display_name': self.attribute_type.replace('_', ' ').title(),
                    'value': employee['user']['email'],
                    'source_id': employee['id'],
                    'active': True,
                    'detail': {
                        'user_id': employee['user_id'],
                        'employee_code': employee['code'],
                        'full_name': employee['user']['full_name'],
                        'location': employee['location'],
                        'department': employee['department']['name'] if employee['department'] else None,
                        'department_id': employee['department_id'],
                        'department_code': employee['department']['code'] if employee['department'] else None
                    }
                })

            self.bulk_create_or_update_expense_attributes(employee_attributes, True)



from .base import Base


class Departments(Base):
    """Class for Departments APIs."""

    def __init__(self):
        Base.__init__(self, attribute_type='DEPARTMENT', query_params={'is_enabled': 'eq.true'})
    
    def search_departments(self, query_params):
        """
        Get of departments filtered on query parameters
        :return: Generator
        """
        query_params['order'] = 'updated_at.desc'
        return self.connection.list_all(query_params)



from .base import Base

class TaxGroups(Base):
    """
    Class for Tax Groups API
    """

    def __init__(self):
        Base.__init__(self, attribute_type='TAX_GROUP', query_params={'order': 'id.asc'})

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        generator = self.get_all_generator()
        for items in generator:
            tax_attributes = []
            for tax_group in items['data']:
                    tax_attributes.append({
                        'attribute_type': 'TAX_GROUP',
                        'display_name': 'Tax Group',
                        'value': tax_group['name'],
                        'source_id': tax_group['id'],
                        'active': tax_group['is_enabled'],
                        'detail': {
                            'tax_rate': tax_group['percentage']
                        }
                    })

            self.bulk_create_or_update_expense_attributes(tax_attributes, True)



from typing import List
import base64
import requests

from .base import Base

class Files(Base):
    """
    Class for File API
    """

    def get_as_base64(self, url):
        return base64.b64encode(requests.get(url).content).decode('ascii')

    def bulk_generate_file_urls(self, data: List[dict]) ->  List[dict]:
        """
        Bulk generate download and upload URLs
        """

        payload = {
            'data': data
        }

        attachments = self.connection.bulk_generate_file_urls(payload=payload)['data']

        for attachment in attachments:
            attachment['download_url'] = self.get_as_base64(attachment['download_url'])
        
        return attachments
        




from .base import Base


class ExpenseCustomFields(Base):
    """Class for Expense Custom Fields APIs."""

    def sync(self):
        """
        Syncs the latest API data to DB.
        """
        query_params = {'order': 'updated_at.desc', 'is_custom': 'eq.true', 'type': 'eq.SELECT', 'is_enabled': 'eq.true'}
        generator = self.connection.list_all(query_params)

        for items in generator:
            for row in items['data']:
                attributes = []
                count = 1
                attribute_type = row['field_name'].upper().replace(' ', '_')
                for option in row['options']:
                    attributes.append({
                        'attribute_type': attribute_type,
                        'display_name': row['field_name'],
                        'value': option,
                        'active': True,
                        'source_id': 'expense_custom_field.{}.{}'.format(row['field_name'].lower(), count),
                        'detail': {
                            'custom_field_id': row['id'],
                            'placeholder': row['placeholder']
                        }
                    })
                    count = count + 1
                self.attribute_type = attribute_type
                self.bulk_create_or_update_expense_attributes(attributes, True)



import setuptools

with open('../README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='fyle-integrations-platform-connector',
    version='1.22.1',
    author='Shwetabh Kumar',
    author_email='shwetabh.kumar@fyle.in',
    description='A common platform connector for all the Fyle Integrations to interact with Fyle Platform APIs',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['fyle', 'api', 'python', 'integration', 'platform', 'connector'],
    url='https://github.com/fylein/fyle-integrations-platform-connector',
    packages=setuptools.find_packages(),
    install_requires=[
        'fyle_accounting_mappings>=1.7.0',
        'fyle>=v0.28.0'
    ],
    classifiers=[
        'Topic :: Internet :: WWW/HTTP',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ]
)
